#!/usr/bin/python

# Anne S. Warlaumont

# A simple computational model of the social feedback loop facilitating child speech development
# My adaptation to do something in between Rescorla-Wagner matching and the too-fast-to-converge traditional Q-learning

import random

def gradualRL(ASDadult,ASDvol): # ASD can either be 0 (which models typical development) or 1 (which models autism)
    
        alpha=.000008 # fixed parameter that determines learning rate
        numdays=365*3 # 3 years times 365 days per year
        # If alpha is set to .2 and numdays is set to 1, a more realistic child contingency on adult is achieved. Learning happens unrealistically quickly though.
    
        # Initialize the probability of the child producing a speech-related sound and the probability of the child producing a non-speech-related sound:
        Psp=.5
        Pnsp=.5
        
        # Set the probability of adult response given that the child voc is speech-related or not speech-related.
        # The values below are drawn from LENA data where ASD and TD groups are matched on age, gender, and maternal education.
        if ASDadult==0:
            Prsp= .209
            Prnsp= .144
        else:
            Prsp= .197
            Prnsp= .148
        
        # Set the number of vocalizations per day
        # The values below are drawn from LENA data where ASD and TD groups are matched on age, gender, and maternal education.
        if ASDvol==0:
            vocsperday=2592
        else:
            vocsperday=2123
        numvocs=vocsperday*numdays # total number of vocalizations in the simulation
        
        # Initialize the records of the child's and adult's behaviors:
        sprec=[0]*numvocs # this will store a record of all the child vocalization types produced
        resprec=[0]*numvocs # this will store a record of whether or not each child vocalization received a response
        nsp=0 # the total number of speech-related vocalizations produced
        nnsp=0 # the total number of non-speech-related vocalizations produced
        lastresptosp = False; # Whether or not the previous child speech-related vocalization was responded to
        n_c_sp_r_sp = 0 # the number of child speech-related vocalizations when the previous speech-related vocalization got a response
        n_c_r_sp = 0 # the number of child vocalizations of any type when the previous speech-related vocalization got a response 
        n_c_sp_nor_sp = 0 # the number of child speech-related vocalizations when the previous speech-related vocalization did not get a response
        n_c_nor_sp = 0 # the number of child vocalizations of any type when the previous speech-related vocalization did not get a response
        
        # Run the simulation
        for voc in range(numvocs):
        
            # Get the child behavior:
            rc=random.random()
            if rc <= Psp/(Psp+Pnsp):
                sp=1
            else:
                sp=0
                
            # Record keeping:
            if lastresptosp:
                n_c_r_sp += 1 # number of child vocalizations when the prevous child speech-related vocalization did receive a response
                if sp==1:
                    nsp+=1 # number of child vocalizations that were speech-related
                    n_c_sp_r_sp += 1 # number of child speech-related vocalizations when the previous child speech-related vocalization did receive a response
                else: 
                    nnsp+=1 # number of child vocalizations that were not speech-related
            else:
                n_c_nor_sp += 1 # number of child vocalizations when the previous child speech-related vocalization did NOT receive a response
                if sp==1:
                    nsp+=1 # number of child vocalizations that were speech-related
                    n_c_sp_nor_sp += 1 # number of child speech-related vocalizations when the previous child speech-related vocalization did NOT receive a response
                else:
                    nnsp+=1 # number of child vocalizations that were not speech-related
        
            # Get the adult behavior:
            ra=random.random()
            if (sp==1 and ra<=Prsp) or (sp==0 and ra<=Prnsp):
                resp=1
            else:
                resp=0
            sprec[voc]=sp
            resprec[voc]=resp
            
            # Record keeping:
            if (sp==1):
                # Reset the record of whether the last speech-related child voc received a response
                if (resp==1):
                    lastresptosp = True
                else:
                    lastresptosp = False
            
            # Learning:
            if sp==1 and resp==1:
                Psp=Psp+alpha
                sumPspPnsp = Psp + Pnsp
                Psp=Psp/sumPspPnsp
                Pnsp=Pnsp/sumPspPnsp
            elif sp==0 and resp==1:
                Pnsp=Pnsp+alpha
                sumPspPnsp = Psp + Pnsp
                Pnsp=Pnsp/sumPspPnsp
                Psp=Psp/sumPspPnsp
        
        # Record keeping:
        lastdaysprate=float(sum(sprec[(numvocs-vocsperday+1):numvocs]))/vocsperday
        childcontonadult = n_c_sp_r_sp/(n_c_r_sp+.01) - (n_c_sp_nor_sp)/(n_c_nor_sp+.01)
        
        
        print
        print "Psp: ", Psp
        print "Pnsp: ", Pnsp
        #print "nsp: ", nsp
        #print "nnsp: ", nnsp
        print "proportion speech-related during the last day: ", lastdaysprate
        print "child contingency on adult response: ", childcontonadult
    
        return Psp, Pnsp, nsp, nnsp, lastdaysprate, childcontonadult
    	
				